/*******************************************************************************
 * Copyright (c) 2001, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.soda.sat.core.internal.util;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.List;

import org.eclipse.soda.sat.core.internal.nls.Messages;
import org.eclipse.soda.sat.core.util.ServiceReferenceUtility;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogService;

/**
 * The <code>ConsoleLog</code> is an implementation of the OSGi defined
 * interface <code>LogService</code> that logs to the console.
 */
class ConsoleLog extends Object implements LogService {
	//
	// Static Fields
	//

	// Externalized String Keys
	private static final String LOG_LEVEL_DEBUG_KEY = "Common.LogLevelDebug";  //$NON-NLS-1$
	private static final String LOG_LEVEL_ERROR_KEY = "Common.LogLevelError";  //$NON-NLS-1$
	private static final String LOG_LEVEL_INFO_KEY = "Common.LogLevelInfo";  //$NON-NLS-1$
	private static final String LOG_LEVEL_UNKNOWN_KEY = "Common.Unknown";  //$NON-NLS-1$
	private static final String LOG_LEVEL_WARNING_KEY = "Common.LogLevelWarning";  //$NON-NLS-1$

	// Externalized String Values
	private static final String LOG_LEVEL_DEBUG_VALUE = Messages.getString(ConsoleLog.LOG_LEVEL_DEBUG_KEY);
	private static final String LOG_LEVEL_ERROR_VALUE = Messages.getString(ConsoleLog.LOG_LEVEL_ERROR_KEY);
	private static final String LOG_LEVEL_INFO_VALUE = Messages.getString(ConsoleLog.LOG_LEVEL_INFO_KEY);
	private static final String LOG_LEVEL_UNKNOWN_VALUE = Messages.getString(ConsoleLog.LOG_LEVEL_UNKNOWN_KEY);
	private static final String LOG_LEVEL_WARNING_VALUE = Messages.getString(ConsoleLog.LOG_LEVEL_WARNING_KEY);

	// Misc
	private static final int DEFAULT_BUFFER_SIZE = 1024;  // 1K
	private static final String LINE_SEPARATOR = System.getProperty("line.separator");  //$NON-NLS-1$

	//
	// Instance Fields
	//

	private StringBuffer buffer;

	//
	// Constructors
	//

	/**
	 * Construct a new ConsoleLog.
	 */
	ConsoleLog() {
		super();
		setBuffer(new StringBuffer(ConsoleLog.DEFAULT_BUFFER_SIZE));
	}

	//
	// Instance Methods
	//

	/**
	 * Format the log message for writing to the console.
	 *
	 * @param reference  The service reference.
	 * @param level      The log level.
	 * @param message    The message string to format.
	 *
	 * @return String
	 */
	private String formatLogMessage(ServiceReference reference, int level, String message) {
		StringBuffer buffer = getBuffer();
		int count = buffer.length();
		buffer.delete(0, count);  // Reset the buffer.

		String prefix = getPrefix(level);
		buffer.append(prefix);

		buffer.append(' ');
		buffer.append(message);

		if (reference != null) {
			buffer.append(", service names=[");  //$NON-NLS-1$
			List/*<String>*/ names = getServiceNames(reference);
			Iterator/*<String>*/ iterator = names.iterator();

			while (iterator.hasNext() == true) {
				String name = (String) iterator.next();
				buffer.append(name);

				if (iterator.hasNext() == true) {
					buffer.append(", ");  //$NON-NLS-1$
				}
			}

			buffer.append(']');
		}

		buffer.append(ConsoleLog.LINE_SEPARATOR);

		String logMessage = buffer.toString();
		return logMessage;
	}

	/**
	 * Private buffer getter.
	 *
	 * @return The StringBuffer.
	 */
	private StringBuffer getBuffer() {
		return buffer;
	}

	/**
	 * Get the log level description.
	 *
	 * @param level  The log level.
	 *
	 * @return A string description of the log level.
	 */
	private String getLevelText(int level) {
		String text;

		switch (level) {
			case LogService.LOG_ERROR:
				text = ConsoleLog.LOG_LEVEL_ERROR_VALUE;
				break;

			case LogService.LOG_WARNING:
				text = ConsoleLog.LOG_LEVEL_WARNING_VALUE;
				break;

			case LogService.LOG_INFO:
				text = ConsoleLog.LOG_LEVEL_INFO_VALUE;
				break;

			case LogService.LOG_DEBUG:
				text = ConsoleLog.LOG_LEVEL_DEBUG_VALUE;
				break;

			default:
				text = ConsoleLog.LOG_LEVEL_UNKNOWN_VALUE;
				break;
		}

		return text;
	}

	/**
	 * Get the output stream used when logging to a message.
	 *
	 * @param level  The log level.
	 *
	 * @return <code>System.out</code> for information and debug messages, and
	 * <code>System.err</code> for error and warning messages.
	 */
	private OutputStream getOutputStream(int level) {
		OutputStream stream;

		if (level == LogService.LOG_INFO || level == LogService.LOG_DEBUG) {
			stream = System.out;
		} else {
			stream = System.err;
		}

		return stream;
	}

	/**
	 * Get the message prefix.
	 *
	 * @param level  The log level.
	 *
	 * @return The message prefix string.
	 */
	private String getPrefix(int level) {
		String levelText = getLevelText(level);

		StringBuffer buffer = new StringBuffer(10);
		buffer.append('[');
		buffer.append(levelText);
		buffer.append(']');

		String prefix = buffer.toString();
		return prefix;
	}

	/**
	 * Query the service names represented by a service reference.
	 *
	 * @param reference  The service reference to be queried.
	 *
	 * @return The names of the services represented by a service reference.
	 */
	private List/*<String>*/ getServiceNames(ServiceReference reference) {
		ServiceReferenceUtility utility = ServiceReferenceUtility.getInstance();
		List/*<String>*/ names = utility.getServiceNames(reference);
		return names;
	}

	/**
	 * @see org.osgi.service.log.LogService#log(int, String)
	 */
	public void log(int level, String message) {
		log(null, level, message, null);
	}

	/**
	 * @see org.osgi.service.log.LogService#log(int, String, Throwable)
	 */
	public void log(int level, String message, Throwable throwable) {
		log(null, level, message, throwable);
	}

	/**
	 * @see org.osgi.service.log.LogService#log(ServiceReference, int, String)
	 */
	public void log(ServiceReference reference, int level, String message) {
		log(reference, level, message, null);
	}

	/**
	 * @see org.osgi.service.log.LogService#log(ServiceReference, int, String, Throwable)
	 */
	public void log(ServiceReference reference, int level, String message, Throwable throwable) {
		OutputStream stream = getOutputStream(level);

		synchronized (this) {
			String logMessage = formatLogMessage(reference, level, message);
			byte[] bytes = logMessage.getBytes();

			try {
				stream.write(bytes);
				stream.flush();

				if (throwable != null) {
					PrintStream printStream = new PrintStream(stream);
					throwable.printStackTrace(printStream);
					printStream.flush();
				}
			} catch (IOException exception) {
				exception.printStackTrace();
			}
		}
	}

	/**
	 * Private buffer setter.
	 *
	 * @param buffer  The StringBuffer.
	 */
	private void setBuffer(StringBuffer buffer) {
		this.buffer = buffer;
	}
}